// $Id$
//
// \file TestUtility.cpp
//
// Copyright (C) 2012, ARM Research Labs, LLC.
// See www.armresearch.com for the copyright terms.
//
// This is the unit test for the class Utility.
//

#include <iostream>
#include <sstream>
#include <stdexcept>

#include <cstdlib>
#include <cstring>
#include <cerrno>

#include "Utility.hpp"

/// Output error.
#define Error(msg)                                                         \
    {                                                                      \
        std::cerr << "In file " << __FILE__ << ":" << __LINE__ << ":  ";   \
        std::cerr << msg;                                                  \
    }

/// Exit with error.
#define ErrorExit(msg)                                                     \
    {                                                                      \
        Error(msg)                                                         \
        std::exit(-1);                                                     \
    }

bool
TestReplaceXmlAttribute() {

    std::string Content;
    std::string ExpectedContent;
    std::string ElementName = "TestElement";
    std::string AttributeName = "TestAttribute";
    std::string AttributeNewValue = "NewValue";

    // Test with valid input.
    std::string OriginalContent;

    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute='testmode' authentication='setuptestingonly'/>\n"
      "<TestXXX XXXAttribute='testmode' authentication='setuptestingonly'/>\n"
      "</snf>\n";

    OriginalContent = Content;

    ExpectedContent = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute='NewValue' authentication='setuptestingonly'/>\n"
      "<TestXXX XXXAttribute='testmode' authentication='setuptestingonly'/>\n"
      "</snf>\n";

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);

    } catch (std::exception &e) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception thrown with valid input:  ";
	Temp += e.what();
	Temp += "\n\nOriginal Content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    }

    if (Content != ExpectedContent) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Incorrect result with valid input.  Content after replacement:\n\n";
	Temp += Content;
	Temp += "\n\nExpected:\n\n";
	Temp += ExpectedContent + "\n";
	Error(Temp);
	return false;

    }

    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\n\t= \n\t\t  'testmode' authentication='setuptestingonly'>\n"
      "</TestElement>"
      "</snf>\n";

    OriginalContent = Content;

    ExpectedContent = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\n\t= \n\t\t  'NewValue' authentication='setuptestingonly'>\n"
      "</TestElement>"
      "</snf>\n";

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);

    } catch (std::exception &e) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception thrown with valid input:  ";
	Temp += e.what();
	Temp += "\n\nOriginal Content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    }

    if (Content != ExpectedContent) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Incorrect result with valid input.  Content after replacement:\n\n";
	Temp += Content;
	Temp += "\n\nExpected:\n\n";
	Temp += ExpectedContent + "\n";
	Error(Temp);
	return false;

    }

    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute=\"testmode\" TestXXAttribute='setuptestingonly'/>\n"
      "</snf>\n";

    OriginalContent = Content;

    ExpectedContent = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute=\"NewValue\" TestXXAttribute='setuptestingonly'/>\n"
      "</snf>\n";

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);

    } catch (std::exception &e) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception thrown with valid input:  ";
	Temp += e.what();
	Temp += "\n\nOriginal Content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    }

    if (Content != ExpectedContent) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Incorrect result with valid input.  Content after replacement:\n\n";
	Temp += Content;
	Temp += "\n\nExpected:\n\n";
	Temp += ExpectedContent + "\n";
	Error(Temp);
	return false;

    }

    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"testmode\" authentication='setuptestingonly'/>\n"
      "</snf>\n";

    OriginalContent = Content;

    ExpectedContent = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"NewValue\" authentication='setuptestingonly'/>\n"
      "</snf>\n";

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);

    } catch (std::exception &e) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception thrown with valid input:  ";
	Temp += e.what();
	Temp += "\n\nOriginal Content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    }

    if (Content != ExpectedContent) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Incorrect result with valid input.  Content after replacement:\n\n";
	Temp += Content;
	Temp += "\n\nExpected:\n\n";
	Temp += ExpectedContent + "\n";
	Error(Temp);
	return false;

    }

    // Element in comment.
    Content = "<!-- License file created by SNFIdentity -->\n"
      "<!-- This is a comment\n"
      "<TestElement TestAttribute =  \"testmode\" authentication='setuptestingonly'/>\n"
      "-->\n"
      "<snf>\n"
      "<TestElement TestAttribute =  \"testmode\" authentication='setuptestingonly'/>\n"
      "</snf>\n";
    OriginalContent = Content;

    ExpectedContent = "<!-- License file created by SNFIdentity -->\n"
      "<!-- This is a comment\n"
      "<TestElement TestAttribute =  \"testmode\" authentication='setuptestingonly'/>\n"
      "-->\n"
      "<snf>\n"
      "<TestElement TestAttribute =  \"NewValue\" authentication='setuptestingonly'/>\n"
      "</snf>\n";

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);

    } catch (std::exception &e) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Unexpected exception thrown with valid input:  ";
	Temp += e.what();
	Temp += "\n\nOriginal Content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    }

    if (Content != ExpectedContent) {
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Incorrect result with valid input.  Content after replacement:\n\n";
	Temp += Content;
	Temp += "\n\nExpected:\n\n";
	Temp += ExpectedContent + "\n";
	Error(Temp);
	return false;

    }

    // Test with invalid input.
    // Duplicate element.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"testmode\" authentication='setuptestingonly'/>\n"
      "<TestElement TestAttribute\t\n=  \"testmode\" authentication='setuptestingonly'/>\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with two <TestElement> elements.  Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Missing element.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with missing <TestElement> element.  Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Malformed element.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"testmode\" authentication='setuptestingonly'\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with malformed <TestElement> element.  Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Duplicate attribute.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"testmode\" TestAttribute='setuptestingonly'/>\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with two TestAttribute attributes.  Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Attribute in another element
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement XXXTestAttribute='setuptestingonly'/>\n"
      "<ATestElement TestAttribute\t\n=  \"testmode\"/>\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with TestAttribute attribute in another element.  "
	  "Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Missing attribute.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement XXXTestAttribute\t\n=  \"testmode\" YYYTestAttribute='setuptestingonly'/>\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with missing TestAttribute attribute.  Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    // Malformed attribute value.
    Content = "<!-- License file created by SNFIdentity-->\n"
      "<snf>\n"
      "<TestElement TestAttribute\t\n=  \"testmode' />\n"
      "</snf>\n";
    OriginalContent = Content;

    try {

	Utility::ReplaceXmlAttribute(&Content, ElementName, AttributeName, AttributeNewValue);
	std::string Temp;

	Temp = "TestReplaceXmlAttribute:  Exception not thrown with malformed TestAttribute value.  "
	  "Original content:\n\n";
	Temp += OriginalContent + "\n";
	Error(Temp);
	return false;

    } catch (std::exception &e) {

    }

    return true;

}

/// Unit tests for Utility.
//
int main(int argc, char* argv[]) {

    try {                                                                       // Catch anything that breaks loose.

        // Test ReplaceXmlAttribute().
        if (!TestReplaceXmlAttribute()) {
            ErrorExit("TestReplaceXmlAttribute() failure.\n");
        }

    }                                                                           // That's all folks.

    catch(std::exception& e) {                                                  // Report any normal exceptions.

        std::cerr << "Utility exception: " << e.what() << std::endl;
        return -1;

    }
    catch(...) {                                                                // Report any unexpected exceptions.

        std::cerr << "Panic! Unknown Exception!" << std::endl;
        return -1;

    }

    return 0;                                                                   // Normally we return zero.
}
